@@ -7,18 +7,11 @@
  * "COPYING" for more details.
  */
 
-typedef enum { 
-	PAUSE_MENU =    0x00000001, 
-	PAUSE_FOCUS =   0x00000002, 
-	PAUSE_POINTER = 0x00000004,
-	PAUSE_PREF =    0x00000008
-} PauseType;
-
 void iterate_game(Game *game);
 void lose_life(Game *game);
 void run_game(Game *game);
 void pause_game(Game *game, PauseType type, gboolean unpause);
-void end_game(Game *game);
+void end_game(Game *game, EndGameStatus status);
 void next_level(Game *game);
 void key_left_pressed(Game *game);
 void key_left_released(Game *game);
@@ -30,3 +23,6 @@ void key_fire2_pressed(Game *game);
 void key_fire2_released(Game *game);
 void mouse_moved(Game *game, int position);
 void new_life(Game *game);
+int process_events(Game *game);
+void destroy_pending_events_list(GList *pending_events);
+int check_newlife_score(Game *game);
