@@ -13,20 +13,26 @@ static void init_leveldata(Game *game);
  * files, and starts the gui */
 int main(int argc, char **argv) {
 	Game game;
-	if(gnome_score_init(PACKAGE) == -1)
-		g_warning("Failed to initialise gnome_score. Is " PACKAGE " installed setgid to the games group?");
+	gboolean show_score_warning = FALSE;
+
+	show_score_warning = (gnome_score_init(PACKAGE) == -1);
 	memset(&game, 0, sizeof(Game));
 
 	srand((unsigned int) clock());
 
 	bindtextdomain(PACKAGE, GNOMELOCALEDIR);
 	textdomain(PACKAGE);
-	gnome_init(PACKAGE, VERSION, argc, argv);
+	gnome_program_init(PACKAGE, VERSION, LIBGNOMEUI_MODULE, argc, argv,
+			GNOME_PARAM_NONE);
 	gui_init(&game, argc, argv);
 	game.flags = load_flags();
 	init_leveldata(&game);
 
 	init_animations();
+
+	if(show_score_warning)
+		gb_warning("Failed to initialise gnome_score. Is " PACKAGE " installed setgid to the games group?");
+
 	gtk_main();
 
 	return 0;
