@@ -16,6 +16,7 @@
 #include "leveldata.h"
 
 static GtkWidget *dialog = NULL;
+static gboolean flags_changed = FALSE;
 static Flags *newflags;
 
 /* Internal data structures */
@@ -23,21 +24,25 @@ typedef struct {
 	Game *game;
 	GtkWidget *frame;
 	GtkWidget *vbox;
-	GtkWidget *clist;
-	GtkWidget *clist_scrollpane;
-	GtkWidget *clist_hbox;
+	GtkListStore *level_list_store;
+	GtkTreeView *level_list_view;
+	GtkWidget *level_list_scrollpane;
+	GtkWidget *level_list_hbox;
 	GtkWidget *button_hbox;
 	GtkWidget *add_button;
 	GtkWidget *remove_button;
 	GtkWidget *fsel;
-	gint clist_row;
+	gchar *level_list_selection;
+	GtkTreeIter level_list_iter;
 } LevelFrame;
 
 /* Internal functions */
-static void init_preferences_box(GuiInfo *gui);
-static void init_game_page(GuiInfo *gui);
-static void init_control_page(GuiInfo *gui);
-static void populate_level_clist(GtkCList *clist, GList *filenames);
+static void init_preferences_box(GuiInfo *gui, GtkNotebook **window_notebook);
+static void init_game_page(GuiInfo *gui, GtkNotebook *window_notebook);
+static void init_control_page(GuiInfo *gui, GtkNotebook *window_notebook);
+static void populate_level_list(LevelFrame *lf, GList *filenames);
+static void apply_preferences(GuiInfo *gui);
+static void set_flags_changed(gboolean status);
 
 /* Callbacks */
 static void cb_pref_destroy(GtkWidget *widget, gpointer data);
@@ -50,18 +55,18 @@ static void cb_key_left(GtkWidget *widget, GdkEventKey *event, gpointer data);
 static void cb_key_right(GtkWidget *widget, GdkEventKey *event, gpointer data);
 static void cb_key_fire1(GtkWidget *widget, GdkEventKey *event, gpointer data);
 static void cb_key_fire2(GtkWidget *widget, GdkEventKey *event, gpointer data);
-static void cb_pref_apply(GtkWidget *propertybox, gint page_num, gpointer data);
-static void cb_pref_help(GtkWidget *propertybox, gint page_num, gpointer data);
-static void cb_level_clist_select(GtkCList *widget, gint row, gint column, GdkEventButton *event, gpointer data);
-static void cb_level_clist_unselect(GtkCList *widget, gint row, gint column, GdkEventButton *event, gpointer data);
-static void cb_level_add(gpointer data);
-static void cb_level_remove(gpointer data);
+static void cb_pref_response(GtkWidget *widget, gint response_id, gpointer data);
+static void cb_level_list_changed(GtkTreeSelection *selection,
+		gpointer data);
+static void cb_level_add(GtkWidget *widget, gpointer data);
+static void cb_level_remove(GtkWidget *widget, gpointer data);
 static void cb_filesel_ok(GtkWidget *w, gpointer data);
 static void cb_filesel_cancel(GtkWidget *w, gpointer data);
 static void cb_checkbox_toggled(GtkWidget *w, gboolean *flag);
 
 void make_preferences_box(GuiInfo *gui) {
-
+	GtkNotebook *window_notebook;
+	
 	/* Don't let the user run two preferences boxes at the same time */
 	if(dialog) {
 		gdk_window_show( dialog->window );
@@ -76,31 +81,49 @@ void make_preferences_box(GuiInfo *gui) {
 
 	newflags = copy_flags(gui->game->flags);
 
-	init_preferences_box(gui);
-	init_game_page(gui); 
-	init_control_page(gui);
-	gtk_widget_show(dialog);
+	init_preferences_box(gui, &window_notebook);
+	init_game_page(gui, window_notebook); 
+	init_control_page(gui, window_notebook);
+	set_flags_changed(FALSE);
+
+	gtk_widget_show_all(dialog);
+}
+
+static void cb_pref_response(GtkWidget *widget, gint response_id,
+		gpointer data) {
+	GuiInfo *gui;
+	gui = (GuiInfo *) data;
+
+	if(response_id == GTK_RESPONSE_APPLY && flags_changed) {
+		apply_preferences(gui);
+	} else if(response_id == GTK_RESPONSE_OK) {
+		if(flags_changed) {
+			apply_preferences(gui);
+		}
+		gtk_widget_destroy(widget);
+	} else if(response_id == GTK_RESPONSE_CLOSE) {
+		gtk_widget_destroy(widget);
+	}
 }
 
 /* Apply has been clicked */
-static void cb_pref_apply(GtkWidget *propertybox, gint page_num, gpointer data) {
-	GuiInfo *gui;
-	gui = (GuiInfo *) data;
+static void apply_preferences(GuiInfo *gui) {
+	//GuiInfo *gui;
+	//gui = (GuiInfo *) data;
 
 	save_flags(newflags);
 	destroy_flags(gui->game->flags);
 	gui->game->flags = newflags;
 	newflags = copy_flags(newflags);
+	set_flags_changed(FALSE);
 }
 
-/* Help has been clicked */
-static void cb_pref_help(GtkWidget *propertybox, gint page_num, gpointer data) {
-	GuiInfo *gui;
-	GtkWidget *mbox;
-
-	gui = (GuiInfo *) data;
-	mbox = gnome_ok_dialog_parented(_("Help hasn't been implimented yet. So go ahead and write some for me :)"), GTK_WINDOW(gui->app));
-	gtk_widget_show(mbox);
+/* Changed the behaviour of the dialog box depending on whether the flags
+ * have been changed since the last commit */
+static void set_flags_changed(gboolean status) {
+	flags_changed = status;
+	gtk_dialog_set_response_sensitive(GTK_DIALOG(dialog),
+			GTK_RESPONSE_APPLY, status);
 }
 
 /* Preferences dialog has been destroyed */
@@ -120,23 +143,27 @@ static void cb_pref_destroy(GtkWidget *widget, gpointer data) {
  */
 
 /* Initialise the actual dialog box, but don't add anything */
-void init_preferences_box(GuiInfo *gui) {
-	dialog = gnome_property_box_new();
-	gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(gui->app));
-	gtk_window_set_title(GTK_WINDOW(dialog),
-			_("GNOME Breakout Preferences"));
+static void init_preferences_box(GuiInfo *gui, GtkNotebook **window_notebook) {
+        dialog = gtk_dialog_new_with_buttons(
+                _("GNOME Breakout Preferences"), GTK_WINDOW(gui->app),
+                0, GTK_STOCK_APPLY,
+		GTK_RESPONSE_APPLY, GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
+                GTK_STOCK_OK, GTK_RESPONSE_OK, NULL);
 
-	gtk_signal_connect(GTK_OBJECT(dialog), "destroy",
-			GTK_SIGNAL_FUNC(cb_pref_destroy), gui);
-	gtk_signal_connect(GTK_OBJECT(dialog), "apply",
-			GTK_SIGNAL_FUNC(cb_pref_apply), gui);
-	gtk_signal_connect(GTK_OBJECT(dialog), "help",
-			GTK_SIGNAL_FUNC(cb_pref_help), gui);
+	g_signal_connect(GTK_OBJECT(dialog), "destroy",
+			G_CALLBACK(cb_pref_destroy), gui);
+	g_signal_connect(GTK_OBJECT(dialog), "response",
+			G_CALLBACK(cb_pref_response), gui);
+
+	*window_notebook = GTK_NOTEBOOK(gtk_notebook_new());
+	gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->vbox),
+        		GTK_WIDGET(*window_notebook));
+	
 	
 }
 
 /* Initialise the "game" section of the preferences box */
-void init_game_page(GuiInfo *gui) {
+void init_game_page(GuiInfo *gui, GtkNotebook *window_notebook) {
 	GtkWidget *gametablabel;
 	GtkWidget *gamehbox;
 	GtkWidget *gamevbox1;
@@ -163,6 +190,13 @@ void init_game_page(GuiInfo *gui) {
 	GtkWidget *difficulty_medium_radio;
 	GtkWidget *difficulty_hard_radio;
 
+	GtkTreeSelection *level_list_selection;
+	GtkCellRenderer *renderer;
+	GtkTreeViewColumn *column;
+
+	/* Reset levelframe */
+	memset(&lf, 0, sizeof(lf));
+
 	/* Vbox and Tablabel - Constructor */
 	gametablabel = gtk_label_new(_("Game"));
 	gamehbox = gtk_hbox_new(FALSE, GNOME_PAD);
@@ -180,40 +214,54 @@ void init_game_page(GuiInfo *gui) {
 	gtk_widget_show(gamevbox2);
 
 	/* Level Frame - Constructor */
-	lf.clist_row = -1;
 	lf.game = gui->game;
 	lf.frame = gtk_frame_new(_("Levels"));
 	lf.vbox = gtk_vbox_new(FALSE, 0);
-	lf.clist = gtk_clist_new(1);
+	lf.level_list_store = gtk_list_store_new(1, G_TYPE_STRING);
+	lf.level_list_view = GTK_TREE_VIEW(gtk_tree_view_new_with_model(
+			GTK_TREE_MODEL(lf.level_list_store)));
 	//lf.warning_label = gtk_label_new(_("Levels cannot be added or removed\nwhile the game is running."));
 	lf.button_hbox = gtk_hbox_new(TRUE, 0);
-	lf.clist_hbox = gtk_hbox_new(TRUE, 0);
+	lf.level_list_hbox = gtk_hbox_new(TRUE, 0);
 	lf.add_button = gtk_button_new_with_label(_("Add"));
 	lf.remove_button = gtk_button_new_with_label(_("Remove"));
 	lf.game = gui->game;
-	lf.clist_scrollpane = gtk_scrolled_window_new(NULL, NULL);
+	lf.level_list_scrollpane = gtk_scrolled_window_new(NULL, NULL);
+	lf.level_list_selection = NULL;
+
+	renderer = gtk_cell_renderer_text_new ();
+	column = gtk_tree_view_column_new_with_attributes ("",
+		renderer, "text", 0, NULL);
+	gtk_tree_view_append_column(lf.level_list_view, column);
 
 	/* Level Frame - Settings */
+	level_list_selection = gtk_tree_view_get_selection(lf.level_list_view);
+	gtk_tree_selection_set_mode(level_list_selection, GTK_SELECTION_SINGLE);
+
 	gtk_frame_set_shadow_type(GTK_FRAME(lf.frame), GTK_SHADOW_ETCHED_IN);
-	gtk_clist_set_shadow_type(GTK_CLIST(lf.clist), GTK_SHADOW_ETCHED_IN);
-	gtk_clist_set_selection_mode(GTK_CLIST(lf.clist), GTK_SELECTION_SINGLE);
-	gtk_clist_set_column_width(GTK_CLIST(lf.clist), 0, 200);
+	//gtk_clist_set_shadow_type(GTK_CLIST(lf.clist), GTK_SHADOW_ETCHED_IN);
+	//gtk_clist_set_column_width(GTK_CLIST(lf.clist), 0, 200);
 	levelfiles = leveldata_titlelist();
-	populate_level_clist(GTK_CLIST(lf.clist), levelfiles);
+	populate_level_list(&lf, levelfiles);
 	g_list_free(levelfiles);
-	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (lf.clist_scrollpane), GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);
+	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (lf.level_list_scrollpane), GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);
+	gtk_tree_view_set_headers_visible(lf.level_list_view, FALSE);
 	
 	/* Level Frame - Signals */
-	gtk_signal_connect(GTK_OBJECT(lf.clist), "select_row", GTK_SIGNAL_FUNC(cb_level_clist_select), (gpointer) &lf);
-	gtk_signal_connect(GTK_OBJECT(lf.clist), "unselect_row", GTK_SIGNAL_FUNC(cb_level_clist_unselect), (gpointer) &lf);
-	gtk_signal_connect_object(GTK_OBJECT(lf.add_button), "clicked", GTK_SIGNAL_FUNC(cb_level_add), (gpointer) &lf);
-	gtk_signal_connect_object(GTK_OBJECT(lf.remove_button), "clicked", GTK_SIGNAL_FUNC(cb_level_remove), (gpointer) &lf);
+	g_signal_connect(G_OBJECT(level_list_selection), "changed", G_CALLBACK(cb_level_list_changed), (gpointer) &lf);
+	/*
+	g_signal_connect_object(GTK_OBJECT(lf.add_button), "clicked", GTK_SIGNAL_FUNC(cb_level_add), (gpointer) &lf, 0);
+	g_signal_connect_object(GTK_OBJECT(lf.remove_button), "clicked", GTK_SIGNAL_FUNC(cb_level_remove), (gpointer) &lf, 0);
+	*/
+	g_signal_connect(GTK_OBJECT(lf.add_button), "clicked", GTK_SIGNAL_FUNC(cb_level_add), (gpointer) &lf);
+	g_signal_connect(GTK_OBJECT(lf.remove_button), "clicked", GTK_SIGNAL_FUNC(cb_level_remove), (gpointer) &lf);
+
 
 	/* Level Frame - Sensitivity */
 	if(gui->game->state != STATE_STOPPED) {
 		gtk_widget_set_sensitive(GTK_WIDGET(lf.add_button), FALSE);
-		gtk_widget_set_sensitive(GTK_WIDGET(lf.clist), FALSE);
-		gtk_widget_set_sensitive(GTK_WIDGET(lf.clist_scrollpane), FALSE);
+		gtk_widget_set_sensitive(GTK_WIDGET(lf.level_list_view), FALSE);
+		gtk_widget_set_sensitive(GTK_WIDGET(lf.level_list_scrollpane), FALSE);
 	}
 	gtk_widget_set_sensitive(GTK_WIDGET(lf.remove_button), FALSE);
 
@@ -221,21 +269,21 @@ void init_game_page(GuiInfo *gui) {
 	gtk_box_pack_start(GTK_BOX(gamevbox1), lf.frame, TRUE, TRUE, GNOME_PAD);
 	gtk_container_add(GTK_CONTAINER(lf.frame), lf.vbox);
 	//gtk_box_pack_start_defaults(GTK_BOX(lf.vbox), lf.warning_label);
-	gtk_box_pack_start(GTK_BOX(lf.vbox), lf.clist_hbox, TRUE, TRUE, GNOME_PAD);
+	gtk_box_pack_start(GTK_BOX(lf.vbox), lf.level_list_hbox, TRUE, TRUE, GNOME_PAD);
 	gtk_box_pack_start(GTK_BOX(lf.vbox), lf.button_hbox, FALSE, FALSE, GNOME_PAD);
-	gtk_container_add(GTK_CONTAINER(lf.clist_scrollpane), lf.clist);
-	gtk_box_pack_start(GTK_BOX(lf.clist_hbox), lf.clist_scrollpane, TRUE, TRUE, GNOME_PAD);
+	gtk_container_add(GTK_CONTAINER(lf.level_list_scrollpane), GTK_WIDGET(lf.level_list_view));
+	gtk_box_pack_start(GTK_BOX(lf.level_list_hbox), lf.level_list_scrollpane, TRUE, TRUE, GNOME_PAD);
 	gtk_box_pack_start(GTK_BOX(lf.button_hbox), lf.add_button, TRUE, TRUE, GNOME_PAD);
 	gtk_box_pack_start(GTK_BOX(lf.button_hbox), lf.remove_button, TRUE, TRUE, GNOME_PAD);
 
 	/* Level Frame - Show */	
 	gtk_widget_show(lf.frame);
 	gtk_widget_show(lf.vbox);
-	gtk_widget_show(lf.clist);
+	gtk_widget_show(GTK_WIDGET(lf.level_list_view));
 	//gtk_widget_show(lf.warning_label);
 	gtk_widget_show(lf.button_hbox);
-	gtk_widget_show(lf.clist_hbox);
-	gtk_widget_show(lf.clist_scrollpane);
+	gtk_widget_show(lf.level_list_hbox);
+	gtk_widget_show(lf.level_list_scrollpane);
 	gtk_widget_show(lf.add_button);
 	gtk_widget_show(lf.remove_button);
 	
@@ -270,11 +318,11 @@ void init_game_page(GuiInfo *gui) {
 	}
 
 	/* Difficulty frame and radio buttons - Signals */
-	gtk_signal_connect(GTK_OBJECT(difficulty_easy_radio), "toggled",
+	g_signal_connect(GTK_OBJECT(difficulty_easy_radio), "toggled",
 			GTK_SIGNAL_FUNC(cb_diff), (gpointer) DIFFICULTY_EASY);
-	gtk_signal_connect(GTK_OBJECT(difficulty_medium_radio), "toggled",
+	g_signal_connect(GTK_OBJECT(difficulty_medium_radio), "toggled",
 			GTK_SIGNAL_FUNC(cb_diff), (gpointer) DIFFICULTY_MEDIUM);
-	gtk_signal_connect(GTK_OBJECT(difficulty_hard_radio), "toggled",
+	g_signal_connect(GTK_OBJECT(difficulty_hard_radio), "toggled",
 			GTK_SIGNAL_FUNC(cb_diff), (gpointer) DIFFICULTY_HARD);
 
 	/* Difficulty frame and radio buttons - Packing */
@@ -307,9 +355,9 @@ void init_game_page(GuiInfo *gui) {
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pause_on_pointer_check), newflags->pause_on_pointer);
 
 	/* Automatic Pause Frame - Signals */
-	gtk_signal_connect(GTK_OBJECT(pause_on_focus_check), "toggled", GTK_SIGNAL_FUNC(cb_checkbox_toggled), &newflags->pause_on_focus);
-	gtk_signal_connect(GTK_OBJECT(pause_on_pointer_check), "toggled", GTK_SIGNAL_FUNC(cb_checkbox_toggled), &newflags->pause_on_pointer);
-	gtk_signal_connect(GTK_OBJECT(pause_on_pref_check), "toggled", GTK_SIGNAL_FUNC(cb_checkbox_toggled), &newflags->pause_on_pref);
+	g_signal_connect(GTK_OBJECT(pause_on_focus_check), "toggled", GTK_SIGNAL_FUNC(cb_checkbox_toggled), &newflags->pause_on_focus);
+	g_signal_connect(GTK_OBJECT(pause_on_pointer_check), "toggled", GTK_SIGNAL_FUNC(cb_checkbox_toggled), &newflags->pause_on_pointer);
+	g_signal_connect(GTK_OBJECT(pause_on_pref_check), "toggled", GTK_SIGNAL_FUNC(cb_checkbox_toggled), &newflags->pause_on_pref);
 
 	/* Automatic Pause Frame - Packing */
 	gtk_box_pack_start_defaults(GTK_BOX(pause_vbox), pause_on_focus_check);
@@ -334,7 +382,7 @@ void init_game_page(GuiInfo *gui) {
 	gtk_box_pack_start(GTK_BOX(gamevbox2), hide_pointer_check,
 			TRUE, FALSE, GNOME_PAD);
 	gtk_widget_show(hide_pointer_check);
-	gtk_signal_connect(GTK_OBJECT(hide_pointer_check), "toggled",
+	g_signal_connect(GTK_OBJECT(hide_pointer_check), "toggled",
 			GTK_SIGNAL_FUNC(cb_checkbox_toggled),
 			&newflags->hide_pointer);
 	
@@ -355,7 +403,7 @@ void init_game_page(GuiInfo *gui) {
 			TRUE);
 
 	/* Bounce Entropy - Signals */
-	gtk_signal_connect(GTK_OBJECT(bounce_entropy_adjustment),
+	g_signal_connect(GTK_OBJECT(bounce_entropy_adjustment),
 			"value_changed", GTK_SIGNAL_FUNC(cb_bounce_entropy),
 			bounce_entropy_sbutton);
 
@@ -373,11 +421,11 @@ void init_game_page(GuiInfo *gui) {
 	gtk_widget_show(bounce_entropy_sbutton);
 
 	/* Add the game vbox to the dialog */
-	gnome_property_box_append_page(GNOME_PROPERTY_BOX(dialog),
+	gtk_notebook_append_page(GTK_NOTEBOOK(window_notebook),
 			gamehbox, gametablabel);
 }
 
-static void init_control_page(GuiInfo *gui) {
+static void init_control_page(GuiInfo *gui, GtkNotebook *window_notebook) {
 	GtkWidget *controlhbox;
 	GtkWidget *controlvbox1;
 	GtkWidget *controlvbox2;
@@ -444,9 +492,9 @@ static void init_control_page(GuiInfo *gui) {
         gtk_widget_show(method_vbox);
         gtk_widget_show(method_keyboard_radio);
         gtk_widget_show(method_mouse_radio);
-        gtk_signal_connect(GTK_OBJECT(method_keyboard_radio), "toggled",
+        g_signal_connect(GTK_OBJECT(method_keyboard_radio), "toggled",
                         GTK_SIGNAL_FUNC(cb_ctrl_key), NULL);
-        gtk_signal_connect(GTK_OBJECT(method_mouse_radio), "toggled",
+        g_signal_connect(GTK_OBJECT(method_mouse_radio), "toggled",
                         GTK_SIGNAL_FUNC(cb_ctrl_mouse), NULL);
 
         /* Bat speed */
@@ -466,7 +514,7 @@ static void init_control_page(GuiInfo *gui) {
         gtk_widget_show(bat_speed_label);
         gtk_widget_show(bat_speed_hbox);
         gtk_widget_show(bat_speed_sbutton);
-        gtk_signal_connect(GTK_OBJECT(bat_speed_adjustment), "value_changed",
+        g_signal_connect(GTK_OBJECT(bat_speed_adjustment), "value_changed",
                         GTK_SIGNAL_FUNC(cb_bat_speed), bat_speed_sbutton);
 
 	/* Keybindings Frame */
@@ -492,7 +540,7 @@ static void init_control_page(GuiInfo *gui) {
 	gtk_widget_show(keyb_left_label);
 	gtk_widget_show(keyb_left_entry);
 	gtk_widget_show(keyb_left_hbox);
-	gtk_signal_connect(GTK_OBJECT(keyb_left_entry), "key_press_event",
+	g_signal_connect(GTK_OBJECT(keyb_left_entry), "key_press_event",
 			GTK_SIGNAL_FUNC(cb_key_left), NULL);
 
 	/* KB: Right */
@@ -509,7 +557,7 @@ static void init_control_page(GuiInfo *gui) {
 	gtk_widget_show(keyb_right_label);
 	gtk_widget_show(keyb_right_entry);
 	gtk_widget_show(keyb_right_hbox);
-	gtk_signal_connect(GTK_OBJECT(keyb_right_entry), "key_press_event",
+	g_signal_connect(GTK_OBJECT(keyb_right_entry), "key_press_event",
 			GTK_SIGNAL_FUNC(cb_key_right), NULL);
 
 	/* KB: Fire1 */
@@ -526,7 +574,7 @@ static void init_control_page(GuiInfo *gui) {
 	gtk_widget_show(keyb_fire1_label);
 	gtk_widget_show(keyb_fire1_entry);
 	gtk_widget_show(keyb_fire1_hbox);
-	gtk_signal_connect(GTK_OBJECT(keyb_fire1_entry), "key_press_event",
+	g_signal_connect(GTK_OBJECT(keyb_fire1_entry), "key_press_event",
 			GTK_SIGNAL_FUNC(cb_key_fire1), NULL);
 
 	/* KB: Fire2 */
@@ -543,11 +591,11 @@ static void init_control_page(GuiInfo *gui) {
 	gtk_widget_show(keyb_fire2_label);
 	gtk_widget_show(keyb_fire2_entry);
 	gtk_widget_show(keyb_fire2_hbox);
-	gtk_signal_connect(GTK_OBJECT(keyb_fire2_entry), "key_press_event",
+	g_signal_connect(GTK_OBJECT(keyb_fire2_entry), "key_press_event",
 			GTK_SIGNAL_FUNC(cb_key_fire2), NULL);
 
 	/* Add it to the dialog */
-	gnome_property_box_append_page(GNOME_PROPERTY_BOX(dialog),
+	gtk_notebook_append_page(GTK_NOTEBOOK(window_notebook),
 			controlhbox, controltablabel);
 }
 
@@ -561,7 +609,7 @@ static void cb_key_left(GtkWidget *widget, GdkEventKey *event, gpointer data) {
 	gtk_entry_set_text(GTK_ENTRY(widget), gdk_keyval_name(event->keyval));
 	newflags->left_key = event->keyval;
 
-	gnome_property_box_changed(GNOME_PROPERTY_BOX (dialog));
+	set_flags_changed(TRUE);
 }
 
 static void cb_key_right(GtkWidget *widget, GdkEventKey *event, gpointer data) {
@@ -571,7 +619,7 @@ static void cb_key_right(GtkWidget *widget, GdkEventKey *event, gpointer data) {
 	gtk_entry_set_text(GTK_ENTRY(widget), gdk_keyval_name(event->keyval));
 	newflags->right_key = event->keyval;
 
-	gnome_property_box_changed(GNOME_PROPERTY_BOX (dialog));
+	set_flags_changed(TRUE);
 }
 
 static void cb_key_fire1(GtkWidget *widget, GdkEventKey *event, gpointer data) {
@@ -581,7 +629,7 @@ static void cb_key_fire1(GtkWidget *widget, GdkEventKey *event, gpointer data) {
 	gtk_entry_set_text(GTK_ENTRY(widget), gdk_keyval_name(event->keyval));
 	newflags->fire1_key = event->keyval;
 
-	gnome_property_box_changed(GNOME_PROPERTY_BOX (dialog));
+	set_flags_changed(TRUE);
 }
 
 static void cb_key_fire2(GtkWidget *widget, GdkEventKey *event, gpointer data) {
@@ -591,7 +639,7 @@ static void cb_key_fire2(GtkWidget *widget, GdkEventKey *event, gpointer data) {
 	gtk_entry_set_text(GTK_ENTRY(widget), gdk_keyval_name(event->keyval));
 	newflags->fire2_key = event->keyval;
 
-	gnome_property_box_changed(GNOME_PROPERTY_BOX (dialog));
+	set_flags_changed(TRUE);
 }
 
 /*
@@ -599,7 +647,7 @@ static void cb_key_fire2(GtkWidget *widget, GdkEventKey *event, gpointer data) {
  */
 static void cb_diff(GtkWidget *widget, gpointer data) {
 	newflags->next_game_difficulty = (Difficulty) data;
-	gnome_property_box_changed(GNOME_PROPERTY_BOX (dialog));
+	set_flags_changed(TRUE);
 }
 
 /*
@@ -608,12 +656,12 @@ static void cb_diff(GtkWidget *widget, gpointer data) {
 static void cb_ctrl_key(GtkWidget *widget, gpointer data) {
 	newflags->mouse_control = FALSE;
 	newflags->keyboard_control = TRUE;
-	gnome_property_box_changed(GNOME_PROPERTY_BOX (dialog));
+	set_flags_changed(TRUE);
 }
 static void cb_ctrl_mouse(GtkWidget *widget, gpointer data) {
 	newflags->mouse_control = TRUE;
 	newflags->keyboard_control = FALSE;
-	gnome_property_box_changed(GNOME_PROPERTY_BOX (dialog));
+	set_flags_changed(TRUE);
 }
 
 /*
@@ -622,81 +670,95 @@ static void cb_ctrl_mouse(GtkWidget *widget, gpointer data) {
 
 static void cb_checkbox_toggled(GtkWidget *w, gboolean *flag) {
 	*flag = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w));
-	gnome_property_box_changed(GNOME_PROPERTY_BOX (dialog));
+	set_flags_changed(TRUE);
 }
 
 static void cb_bat_speed(GtkWidget *widget, gpointer data) {
 	newflags->bat_speed = gtk_spin_button_get_value_as_int(
 			GTK_SPIN_BUTTON(data));
-	gnome_property_box_changed(GNOME_PROPERTY_BOX (dialog));
+	set_flags_changed(TRUE);
 }
 
 static void cb_bounce_entropy(GtkWidget *widget, gpointer data) {
 	newflags->bounce_entropy = gtk_spin_button_get_value_as_int(
 			GTK_SPIN_BUTTON(data));
-	gnome_property_box_changed(GNOME_PROPERTY_BOX (dialog));
+	set_flags_changed(TRUE);
 }
 
-static void populate_level_clist(GtkCList *clist, GList *filenames) {
+static void populate_level_list(LevelFrame *lf, GList *filenames) {
 	GList *curr;
 	gchar *data;
-
+		
 	for(curr = filenames; curr; curr = g_list_next(curr)) {
 		data = (gchar *) curr->data;
-		gtk_clist_append(GTK_CLIST(clist), &data);
+		gtk_list_store_append(lf->level_list_store, &(lf->level_list_iter));
+		gtk_list_store_set(lf->level_list_store, &(lf->level_list_iter), 0, data, -1);
 	}
 }
 
-static void cb_level_clist_select(GtkCList *widget, gint row, gint column, GdkEventButton *event, gpointer data) {
+static void cb_level_list_changed(GtkTreeSelection *selection,
+		gpointer data) {
 	LevelFrame *lf = (LevelFrame *) data;
-	if(lf->game->state == STATE_STOPPED) {
-		gtk_widget_set_sensitive(GTK_WIDGET(lf->remove_button), TRUE);
+	GtkTreeModel *model;
+	if(gtk_tree_selection_get_selected(selection, &model, &(lf->level_list_iter))) {
+		if(lf->game->state == STATE_STOPPED) {
+			gtk_widget_set_sensitive(GTK_WIDGET(lf->remove_button), TRUE);
+		}
+
+		if (lf->level_list_selection) {
+			g_free(lf->level_list_selection);
+			lf->level_list_selection = NULL;
+		}
+
+		gtk_tree_model_get(model, &(lf->level_list_iter), 0, &(lf->level_list_selection), -1);
+	} else {
+		gtk_widget_set_sensitive(GTK_WIDGET(lf->remove_button), FALSE);
+		lf->level_list_selection = NULL;
 	}
-	lf->clist_row = row;
 }
 
-static void cb_level_clist_unselect(GtkCList *widget, gint row, gint column, GdkEventButton *event, gpointer data) {
-	LevelFrame *lf = (LevelFrame *) data;
-	gtk_widget_set_sensitive(GTK_WIDGET(lf->remove_button), FALSE);
-	lf->clist_row = -1;
-}
-
-static void cb_level_add(gpointer data) {
+static void cb_level_add(GtkWidget *widget, gpointer data) {
 	LevelFrame *lf = (LevelFrame *) data;
 
 	if(lf->game->state == STATE_STOPPED) {
 		lf->fsel = gtk_file_selection_new ("Add Levelfile");
 
 		/* Connect Signals */
-		gtk_signal_connect(GTK_OBJECT(lf->fsel), "destroy", (GtkSignalFunc) cb_filesel_cancel, data);
-		gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION (lf->fsel)->cancel_button), "clicked", (GtkSignalFunc) cb_filesel_cancel, data);
+		g_signal_connect(GTK_OBJECT(lf->fsel), "destroy", (GtkSignalFunc) cb_filesel_cancel, data);
+		g_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION (lf->fsel)->cancel_button), "clicked", (GtkSignalFunc) cb_filesel_cancel, data);
 		
-		gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION (lf->fsel)->ok_button), "clicked", (GtkSignalFunc) cb_filesel_ok, data);
+		g_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION (lf->fsel)->ok_button), "clicked", (GtkSignalFunc) cb_filesel_ok, data);
 
 		/* Set defaults */
 		gtk_file_selection_set_filename(GTK_FILE_SELECTION(lf->fsel), "*.gbl");
 
 		/* Show the selector */
 		gtk_widget_show(lf->fsel);
-	}		
+	} else {
+		gui_warning("Cannot add or remove levels while the game is running");
+	}
+	set_flags_changed(TRUE);
 }
 
-static void cb_level_remove(gpointer data) { 
+static void cb_level_remove(GtkWidget *widget, gpointer data) {
 	LevelFrame *lf = (LevelFrame *) data;
 	gchar *text;
 	gchar *filename;
 
 	if(lf->game->state == STATE_STOPPED) {
-		g_assert(lf->clist_row != -1);
-		g_assert(gtk_clist_get_text(GTK_CLIST(lf->clist), lf->clist_row, 0, &text));
-		filename = leveldata_remove(text);
+		g_assert(lf->level_list_selection);
+
+		filename = leveldata_remove(lf->level_list_selection);
 		remove_flags_levelfile(newflags, filename);
 		g_free(filename);
-		gtk_clist_remove(GTK_CLIST(lf->clist), lf->clist_row);	
+
+		gtk_list_store_remove(lf->level_list_store, &(lf->level_list_iter));
+		g_free(lf->level_list_selection);
+		lf->level_list_selection = NULL;
 	} else {
 		gui_warning("Cannot add or remove levels while the game is running");
 	}
-	gnome_property_box_changed(GNOME_PROPERTY_BOX (dialog));
+	set_flags_changed(TRUE);
 }
 
 static void cb_filesel_ok(GtkWidget *w, gpointer data) {
@@ -704,11 +766,12 @@ static void cb_filesel_ok(GtkWidget *w, gpointer data) {
 	gchar *filename;
 	gchar *title;
 
-	filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION (lf->fsel));
+	filename = (gchar *) gtk_file_selection_get_filename (GTK_FILE_SELECTION (lf->fsel));
 
 	if(lf->game->state == STATE_STOPPED) {
 		if((title = leveldata_add(filename))) {
-			gtk_clist_append(GTK_CLIST(lf->clist), &title);
+			gtk_list_store_append(lf->level_list_store, &(lf->level_list_iter));
+			gtk_list_store_set(lf->level_list_store, &(lf->level_list_iter), 0, title, -1);
 			add_flags_levelfile(newflags, filename);
 		}
 	} else {
@@ -717,7 +780,7 @@ static void cb_filesel_ok(GtkWidget *w, gpointer data) {
 
 	gtk_widget_destroy(GTK_WIDGET(lf->fsel));
 	lf->fsel = NULL;
-	gnome_property_box_changed(GNOME_PROPERTY_BOX (dialog));
+	set_flags_changed(TRUE);
 }
 
 static void cb_filesel_cancel(GtkWidget *w, gpointer data) {
