@@ -7,13 +7,10 @@
  * "COPYING" for more details.
  */
 
-/* The way the game ended */
-typedef enum { ENDGAME_WIN, ENDGAME_LOSE, ENDGAME_MENU } EndGameStatus;
-
 void gui_init(Game *game, int argc, char **argv);
 void add_to_canvas(Entity *entity);
 void remove_from_canvas(Entity *entity);
-void update_gnome(Game *game);
+void gui_update_game(Game *game);
 void process_gnome_events(void);
 void update_canvas_position(Entity *entity);
 void update_canvas_animation(Entity *entity);
@@ -21,6 +18,7 @@ void gui_begin_game(void);
 void gui_end_game(EndGameStatus status);
 gint get_mouse_x_position(void);
 void gui_warning(gchar *format, ...);
+void gui_error(gchar *format, ...);
 
 /* This holds pointers to objects in the currently running GNOME app. It should
  * only be used in gui.c and gui-callbacks.c */
