@@ -632,7 +632,7 @@ static gint verify_raw_data(gint *data, size_t n, gint max) {
 RawLevel *leveldata_get(gint level_num) {
 	GList *curr;
 	gint i;
-	g_assert(level_num <= num_levels);
+	g_assert(level_num < num_levels);
 
 	for(i = 0, curr = levels; i < level_num && curr; i++, curr = g_list_next(curr));
 	g_assert(curr);
